@@ -715,6 +715,29 @@ INSTANTIATE_TEST_SUITE_P(
         "spv.coopvecTraining_Error.comp",
         "spv.intcoopmat.comp",
         "spv.intrinsicsInteractWithCoopMat.comp",
+        "spv.longVector.comp",
+        "spv.longVectorMatMul.comp",
+        "spv.longVectorNotReserved.comp",
+        "spv.longVectorSpecConst.comp",
+        "spv.longVectorOperators.comp",
+        "spv.longVectorBuiltins.comp",
+        "spv.longVectorBuiltinsfp16.comp",
+        "spv.longVectorBuiltinsfp64.comp",
+        "spv.longVectorBuiltinsint8.comp",
+        "spv.longVectorBuiltinsint16.comp",
+        "spv.longVectorBuiltinsint32.comp",
+        "spv.longVectorBuiltinsint64.comp",
+        "spv.longVectorBuiltinsuint8.comp",
+        "spv.longVectorBuiltinsuint16.comp",
+        "spv.longVectorBuiltinsuint32.comp",
+        "spv.longVectorBuiltinsuint64.comp",
+        "spv.longVectorBuiltinsBitcast.comp",
+        "spv.longVectorBuiltinsError.comp",
+        "spv.longVectorBuiltinsSpecConst.comp",
+        "spv.longVectorBuiltinsSpecConstint32.comp",
+        "spv.longVectorBuiltinsSubgroup.comp",
+        "spv.longVectorBuiltins.frag",
+        "spv.longVectorStorage.comp",
         "spv.replicate.comp",
         "spv.replicatespec.comp",
         "spv.atomicAdd.bufferReference.comp",
@@ -828,6 +851,18 @@ INSTANTIATE_TEST_SUITE_P(
         "spv.nv.lss-spheregeomcap.rgen",
         "spv.nv.lss-lssgeomcap.rgen",
 
+        // SPV_EXT_descriptor_heap
+
+        "spv.descriptorHeap.AtomicImage.comp",
+        "spv.descriptorHeap.AtomicImage.error.comp",
+        "spv.descriptorHeap.Buffer.comp",
+        "spv.descriptorHeap.DebugPrintf.comp",
+        "spv.descriptorHeap.HeapNotArray.error.comp",
+        "spv.descriptorHeap.HeapStrideNotHeap.error.comp",
+        "spv.descriptorHeap.HeapStrideNotPow2.error.comp",
+        "spv.descriptorHeap.PushConstant.comp",
+        "spv.descriptorHeap.valid.frag",
+
     })),
     FileNameAsCustomTestSuffix
 );
@@ -1054,7 +1089,8 @@ INSTANTIATE_TEST_SUITE_P(
     "spv.meshShaderRedeclPerViewBuiltins.mesh",
     "spv.meshTaskShader.task",
     "spv.perprimitiveNV.frag",
-    "spv.nvgpushader5.frag"
+    "spv.nvgpushader5.frag",
+    "spv.pushConstantBank.vert",
 })),
 FileNameAsCustomTestSuffix
 );
